/** @file
 * @brief DYNDNS Service class implementation. This class represents the DYNDNS service.
 *
 *
 *
 * @copyright Intra2net AG
 * @license GPLv2
*/

#include "service_dyndns.h"

#include <boost/foreach.hpp>

using namespace std;


/**
 * Default Constructor, needed for object serialization.
 */
ServiceDyndns::ServiceDyndns()
{
}


/**
 * Constructor.
 * @param _hostname The hostname to update
 * @param _login The login name.
 * @param _password The corresponding password.
 */
ServiceDyndns::ServiceDyndns(const string& _protocol, const string& _hostname, const string& _login, const string& _password, const Logger::Ptr& _logger, const int _update_interval, const int _max_updates_within_interval, const int _dns_cache_ttl, const string& _proxy, const int _proxy_port)
{
    if ( _update_interval == -1 )        // If _update_interval is default po::option_desc (not specified via config)
        set_update_interval(0);              // use default protocol value
    else
        set_update_interval(_update_interval);

    if ( _max_updates_within_interval == -1 )
        set_max_updates_within_interval(0);
    else
        set_max_updates_within_interval(_max_updates_within_interval);

    if ( _dns_cache_ttl == -1 )
        set_dns_cache_ttl(60);
    else
        set_dns_cache_ttl(_dns_cache_ttl);

    set_protocol(_protocol);
    set_hostname(_hostname);
    set_login(_login);
    set_password(_password);
    set_logger(_logger);

    // create http helper class
    HTTPHelp = HTTPHelper::Ptr(new HTTPHelper(_logger,_proxy,_proxy_port,_login,_password));

    BaseUrl = assemble_base_url(get_hostname());
}


/**
 * Default destructor
 */
ServiceDyndns::~ServiceDyndns()
{
}


/**
 * Assemble the dyndns update url from the given fqhn
 * @param hostname The fqhn hostname to update IP for.
 * @return The assembled update url without IP.
 */
string ServiceDyndns::assemble_base_url(const string& fqhn) const
{
    string base_url;

    base_url = "https://members.dyndns.org";
    base_url.append("/nic/update?hostname=");
    base_url.append(fqhn);
    base_url.append("&wildcard=NOCHG&mx=NOCHG&backmx=NOCHG&myip=");

    return base_url;
}


/**
 * Performs the Service update.
 * @param ip IP Address to set.
 * @return 0 if all is fine, -1 otherwise.
 */
int ServiceDyndns::perform_update(const std::string& ip)
{
    // the result string if update was successful
    string good = "good ";
    good.append(ip);

    string url = BaseUrl;
    url.append(ip);

    if ( HTTPHelp->is_initialized() )
    {
        // Perform curl operation on given url
        long http_status_code = HTTPHelp->http_get(url);

        get_logger()->print_http_status_code(url,http_status_code);

        // HTTP operation completed successful.
        // Now we have to parse the data received by curl,
        // cause http status code is not significant for dyndns update errors
        if ( http_status_code == 200 )
        {
            // Get the received http data.
            string curl_data = HTTPHelp->get_curl_data();

            if ( curl_data == good )
            {
                return 0;
            }
            else if ( curl_data == "badauth" )
            {
                get_logger()->print_service_not_authorized(url,get_login(),get_password());
            }
            else
            {
                get_logger()->print_update_failure(url, curl_data);
            }
        }
        else
        {
            get_logger()->print_update_failure(url,http_status_code);
        }
    }
    else
    {
        get_logger()->print_httphelper_not_initialized();
    }
    return -1;
}
